/*
 *   To compile:
 *
 *   gcc -fPIC -shared -o rh_spank.so rh_spank.c
 *
 */
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <slurm/spank.h>

#include <sys/utsname.h>

/*
 *  * All spank plugins must define this macro for the
 *   * Slurm plugin loader.
 *    */
SPANK_PLUGIN(rh_spank, 1);

#define RH8 0
#define RH9 1
#define RHANY 2

static int chosen_os = -1;

static int _rh_opt_process (int val,
                                const char *optarg,
                                int remote);

struct spank_option rh8_opt =
    { "rh8", NULL,
      "The job requires rh8 host.", 0, RH8,
      (spank_opt_cb_f) _rh_opt_process
    };

struct spank_option rh9_opt =
    { "rh9", NULL,
      "The job requires rh9 host.", 0, RH9,
      (spank_opt_cb_f) _rh_opt_process
    };

struct spank_option rhany_opt =
    { "rhany", NULL,
      "The job will run well on either rh8 or rh9 host.", 0, RHANY,
      (spank_opt_cb_f) _rh_opt_process
    };

/*
int _rh_print(const char *pr) {
    slurm_info("%s: rh_spank, chosen_os=%d", pr, chosen_os);
}*/

int slurm_spank_init (spank_t sp, int ac, char **av)
{
    struct utsname buffer;
    int sc = spank_context();
    if (sc == S_CTX_ALLOCATOR) {
      spank_option_register (sp, &rh8_opt);
      spank_option_register (sp, &rh9_opt);
      spank_option_register (sp, &rhany_opt);
    }
    if (uname(&buffer) != 0) {
      slurm_error("rh_spank: Failed to check the kernel version.");
      return 1;
    }

    if (buffer.release[0] == '5') {
      chosen_os = RH9;
    } else {
      chosen_os = RH8;
    }
    return (0);
}

static int _rh_opt_process (int val,
                                const char *optarg,
                                int remote) {
    chosen_os = val;
    return (0);
}

int slurm_spank_init_post_opt (spank_t sp, int ac, char **av)
{
  //_rh_print("_rh_slurm_spank_init_post_opt");
  switch (chosen_os) {
    case RH8:
      spank_job_control_setenv(sp, "_SLURM_SPANK_OPTION_rh_spank_rh8","(null)", 1);
      break;
    case RH9:
      spank_job_control_setenv(sp, "_SLURM_SPANK_OPTION_rh_spank_rh9","(null)", 1);
      break;
    case RHANY:
      spank_job_control_setenv(sp, "_SLURM_SPANK_OPTION_rh_spank_rhany","(null)", 1);
      break;
  }
}
